import React, { Component, PropTypes } from 'react';
import { VirtualScroll } from 'react-virtualized';
import classNames from 'classnames';
import moment from 'moment';
import { validParsedDate } from '../utils';
import Month from '../Month';
var style = {
	'root': 'Cal__List__root',
	'scrolling': 'Cal__List__scrolling'
};

var List = function (_Component) {
	babelHelpers.inherits(List, _Component);

	function List(_ref) {
		var months = _ref.months;
		babelHelpers.classCallCheck(this, List);

		var _this = babelHelpers.possibleConstructorReturn(this, Object.getPrototypeOf(List).call(this));

		_initialiseProps.call(_this);

		_this.monthMap = months.map(function (month) {
			return month.date.format('YYYYMM');
		});
		_this.rowHeights = months.map(function (month) {
			return month.height;
		});
		return _this;
	}

	babelHelpers.createClass(List, [{
		key: 'componentDidMount',
		value: function componentDidMount() {
			var vs = this.refs.VirtualScroll;
			var grid = vs && vs.refs.Grid;

			this.scrollEl = grid && grid.refs.scrollingContainer;
		}

		// 	scrollView.removeEventListener('transitionend', onTransitionEnd);
		// }

		// window.requestAnimationFrame(() => {
		// 	scrollView.style.transform = `translate3d(0,${delta}px,0)`;
		// 	scrollView.style.transition = 'transform 0.4s ease';
		//
		// 	scrollView.addEventListener('transitionend', onTransitionEnd);
		// });

	}, {
		key: 'render',
		value: function render() {
			var _props = this.props;
			var height = _props.height;
			var isScrolling = _props.isScrolling;
			var onScroll = _props.onScroll;
			var overscanMonthCount = _props.overscanMonthCount;
			var months = _props.months;
			var rowHeight = _props.rowHeight;
			var selectedDate = _props.selectedDate;
			var today = _props.today;
			var width = _props.width;

			if (!this.initScrollTop) this.initScrollTop = this.getDateOffset(selectedDate && selectedDate.date || today.date);
			if (typeof width == 'string' && width.indexOf('%') !== -1) {
				width = window.innerWidth * parseInt(width.replace('%', ''), 10) / 100; // See https://github.com/bvaughn/react-virtualized/issues/229
			}

			return React.createElement(VirtualScroll, {
				ref: 'VirtualScroll',
				width: width,
				height: height,
				rowCount: months.length,
				rowHeight: this.getRowHeight,
				rowRenderer: this.renderMonth,
				onScroll: onScroll,
				scrollTop: this.initScrollTop,
				className: classNames(style.root, babelHelpers.defineProperty({}, style.scrolling, isScrolling)),
				style: { lineHeight: rowHeight + 'px' },
				overscanRowCount: overscanMonthCount
			});
		}
	}]);
	return List;
}(Component);

List.propTypes = {
	width: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
	height: PropTypes.number,
	rowHeight: PropTypes.number,
	selectedDate: PropTypes.object,
	disabledDates: PropTypes.arrayOf(PropTypes.string),
	disabledDays: PropTypes.arrayOf(PropTypes.number),
	months: PropTypes.arrayOf(PropTypes.object),
	onDaySelect: PropTypes.func,
	onScroll: PropTypes.func,
	overscanMonthCount: PropTypes.number,
	isScrolling: PropTypes.bool,
	today: validParsedDate,
	minDate: validParsedDate,
	maxDate: validParsedDate,
	showOverlay: PropTypes.bool,
	theme: PropTypes.object,
	locale: PropTypes.object
};

var _initialiseProps = function _initialiseProps() {
	var _this2 = this;

	this.getRowHeight = function (_ref2) {
		var index = _ref2.index;

		return _this2.props.months[index].height;
	};

	this.getMonthIndex = function (date) {
		var index = _this2.monthMap.indexOf(moment(date).format('YYYYMM'));

		return index;
	};

	this.getDateOffset = function (date) {
		var index = _this2.getMonthIndex(date);
		var rowHeights = _this2.rowHeights.slice(0, index);
		var offsetTop = rowHeights.reduce(function (a, b) {
			return a + b;
		}, 0);

		return offsetTop;
	};

	this.scrollToDate = function (date) {
		var offset = arguments.length <= 1 || arguments[1] === undefined ? 0 : arguments[1];

		if (_this2.scrollEl) {
			var offsetTop = _this2.getDateOffset(date);
			_this2.scrollTo(offsetTop + offset);
		}
	};

	this.scrollTo = function () {
		var offset = arguments.length <= 0 || arguments[0] === undefined ? 0 : arguments[0];
		var scrollEl = _this2.scrollEl;


		if (scrollEl) {
			// let scrollView = scrollEl.children[0];
			// let currentScrollTop = scrollEl.scrollTop;
			// let delta = -(offset - currentScrollTop);

			// function onTransitionEnd () {
			// 	scrollView.style.transform = 'translate3d(0,0,0)';
			// 	scrollView.style.transition = '';
			scrollEl.scrollTop = offset;
		}
	};

	this.renderMonth = function (_ref3) {
		var index = _ref3.index;
		var isScrolling = _ref3.isScrolling;
		var _props2 = _this2.props;
		var disabledDates = _props2.disabledDates;
		var disabledDays = _props2.disabledDays;
		var locale = _props2.locale;
		var maxDate = _props2.maxDate;
		var minDate = _props2.minDate;
		var months = _props2.months;
		var onDaySelect = _props2.onDaySelect;
		var rowHeight = _props2.rowHeight;
		var selectedDate = _props2.selectedDate;
		var showOverlay = _props2.showOverlay;
		var theme = _props2.theme;
		var today = _props2.today;
		var _months$index = months[index];
		var date = _months$index.date;
		var rows = _months$index.rows;


		return React.createElement(Month, {
			key: 'Month-' + index,
			selectedDate: selectedDate,
			displayDate: date,
			disabledDates: disabledDates,
			disabledDays: disabledDays,
			maxDate: maxDate,
			minDate: minDate,
			onDaySelect: onDaySelect,
			rows: rows,
			rowHeight: rowHeight,
			isScrolling: isScrolling,
			showOverlay: showOverlay,
			today: today,
			theme: theme,
			locale: locale
		});
	};
};

export default List;