import React, { Component, PropTypes } from 'react';
import classNames from 'classnames';
import moment from 'moment';
import debounce from 'lodash/debounce';
import range from 'lodash/range';
import { getScrollSpeed, keyCodes, parseDate, validDate } from './utils';
import defaultLocale from './locale';
import defaultTheme from './theme';
import Today from './Today';
import Header from './Header';
import List from './List';
import Weekdays from './Weekdays';
import Year from './Year';

var containerStyle = {
	'root': 'Cal__Container__root',
	'landscape': 'Cal__Container__landscape',
	'wrapper': 'Cal__Container__wrapper',
	'listWrapper': 'Cal__Container__listWrapper'
};
var dayStyle = {
	'root': 'Cal__Day__root',
	'enabled': 'Cal__Day__enabled',
	'highlighted': 'Cal__Day__highlighted',
	'today': 'Cal__Day__today',
	'disabled': 'Cal__Day__disabled',
	'selected': 'Cal__Day__selected',
	'month': 'Cal__Day__month',
	'year': 'Cal__Day__year',
	'selection': 'Cal__Day__selection',
	'day': 'Cal__Day__day'
};
var style = {
	container: containerStyle,
	day: dayStyle
};

var InfiniteCalendar = function (_Component) {
	babelHelpers.inherits(InfiniteCalendar, _Component);

	function InfiniteCalendar(props) {
		babelHelpers.classCallCheck(this, InfiniteCalendar);


		// Initialize

		var _this = babelHelpers.possibleConstructorReturn(this, Object.getPrototypeOf(InfiniteCalendar).call(this));

		_initialiseProps.call(_this);

		_this.updateLocale(props.locale);
		_this.updateYears(props);
		_this.state = {
			selectedDate: _this.parseSelectedDate(props.selectedDate)
		};
		return _this;
	}

	babelHelpers.createClass(InfiniteCalendar, [{
		key: 'componentDidMount',
		value: function componentDidMount() {
			var _props = this.props;
			var autoFocus = _props.autoFocus;
			var keyboardSupport = _props.keyboardSupport;

			this.node = this.refs.node;
			this.list = this.refs.List;

			if (keyboardSupport && autoFocus) {
				this.node.focus();
			}
		}
	}, {
		key: 'componentWillReceiveProps',
		value: function componentWillReceiveProps(next) {
			var _props2 = this.props;
			var min = _props2.min;
			var minDate = _props2.minDate;
			var max = _props2.max;
			var maxDate = _props2.maxDate;
			var locale = _props2.locale;
			var selectedDate = _props2.selectedDate;


			if (next.locale !== locale) {
				this.updateLocale(next.locale);
			}
			if (next.min !== min || next.minDate !== minDate || next.max !== max || next.maxDate !== maxDate) {
				this.updateYears(next);
			}
			if (next.selectedDate !== selectedDate) {
				this.setState({
					selectedDate: this.parseSelectedDate(next.selectedDate)
				});
			} else if (next.minDate !== minDate || next.maxDate !== maxDate) {
				// Need to make sure the currently selected date is not before the new minDate or after maxDate
				var _selectedDate = this.parseSelectedDate(this.state.selectedDate);
				if (!_selectedDate.isSame(this.state.selectedDate, 'day')) {
					this.setState({
						selectedDate: _selectedDate
					});
				}
			}
		}
	}, {
		key: 'parseSelectedDate',
		value: function parseSelectedDate(selectedDate) {
			if (selectedDate) {
				selectedDate = moment(selectedDate);

				// Selected Date should not be before min date or after max date
				if (selectedDate.isBefore(this._minDate)) {
					return this._minDate;
				} else if (selectedDate.isAfter(this._maxDate)) {
					return this._maxDate;
				}
			}

			return selectedDate;
		}
	}, {
		key: 'updateYears',
		value: function updateYears() {
			var props = arguments.length <= 0 || arguments[0] === undefined ? this.props : arguments[0];

			var min = this._min = moment(props.min);
			var max = this._max = moment(props.max);
			this._minDate = moment(props.minDate);
			this._maxDate = moment(props.maxDate);

			this.years = range(min.year(), max.year() + 1).map(function (year) {
				return Year({ year: year, min: min, max: max, rowHeight: props.rowHeight });
			});
			this.months = [].concat.apply([], this.years);
		}
	}, {
		key: 'updateLocale',
		value: function updateLocale(locale) {
			locale = this.getLocale(locale);
			moment.updateLocale(locale.name, locale);
			moment.locale(locale.name);
		}
	}, {
		key: 'getDisabledDates',
		value: function getDisabledDates(disabledDates) {
			return disabledDates && disabledDates.map(function (date) {
				return moment(date).format('YYYYMMDD');
			});
		}
	}, {
		key: 'getLocale',
		value: function getLocale() {
			var customLocale = arguments.length <= 0 || arguments[0] === undefined ? this.props.locale : arguments[0];

			return Object.assign({}, defaultLocale, customLocale);
		}
	}, {
		key: 'getTheme',
		value: function getTheme() {
			var customTheme = arguments.length <= 0 || arguments[0] === undefined ? this.props.theme : arguments[0];

			return Object.assign({}, defaultTheme, customTheme);
		}
	}, {
		key: 'clearHighlight',
		value: function clearHighlight() {
			if (this.highlightedEl) {
				this.highlightedEl.classList.remove(style.day.highlighted);
				this.highlightedEl = null;
			}
		}
	}, {
		key: 'render',
		value: function render() {
			var _props3 = this.props;
			var className = _props3.className;
			var disabledDays = _props3.disabledDays;
			var height = _props3.height;
			var keyboardSupport = _props3.keyboardSupport;
			var layout = _props3.layout;
			var overscanMonthCount = _props3.overscanMonthCount;
			var minDate = _props3.minDate;
			var maxDate = _props3.maxDate;
			var shouldHeaderAnimate = _props3.shouldHeaderAnimate;
			var showTodayHelper = _props3.showTodayHelper;
			var showHeader = _props3.showHeader;
			var tabIndex = _props3.tabIndex;
			var width = _props3.width;
			var other = babelHelpers.objectWithoutProperties(_props3, ['className', 'disabledDays', 'height', 'keyboardSupport', 'layout', 'overscanMonthCount', 'minDate', 'maxDate', 'shouldHeaderAnimate', 'showTodayHelper', 'showHeader', 'tabIndex', 'width']);

			var disabledDates = this.getDisabledDates(this.props.disabledDates);
			var locale = this.getLocale();
			var theme = this.getTheme();
			var _state = this.state;
			var isScrolling = _state.isScrolling;
			var selectedDate = _state.selectedDate;
			var showToday = _state.showToday;

			var today = this.today = parseDate(moment());

			// Selected date should not be disabled
			if (selectedDate && (disabledDates && disabledDates.indexOf(selectedDate.format('YYYYMMDD')) !== -1 || disabledDays && disabledDays.indexOf(selectedDate.day()) !== -1)) {
				selectedDate = null;
			}

			return React.createElement(
				'div',
				{ tabIndex: tabIndex, onKeyDown: keyboardSupport && this.handleKeyDown, className: classNames(className, style.container.root, babelHelpers.defineProperty({}, style.container.landscape, layout == 'landscape')), style: { color: theme.textColor.default, width: width }, 'aria-label': 'Calendar', ref: 'node' },
				showHeader && React.createElement(Header, { selectedDate: selectedDate, shouldHeaderAnimate: shouldHeaderAnimate, layout: layout, theme: theme, locale: locale, onClick: this.scrollToDate }),
				React.createElement(
					'div',
					{ className: style.container.wrapper },
					React.createElement(Weekdays, { theme: theme }),
					React.createElement(
						'div',
						{ className: style.container.listWrapper },
						showTodayHelper && React.createElement(Today, { scrollToDate: this.scrollToDate, show: showToday, today: today, theme: theme, locale: locale }),
						React.createElement(List, babelHelpers.extends({
							ref: 'List'
						}, other, {
							width: width,
							height: height,
							selectedDate: parseDate(selectedDate),
							disabledDates: disabledDates,
							disabledDays: disabledDays,
							months: this.months,
							onDaySelect: this.onDaySelect,
							onScroll: this.onScroll,
							isScrolling: isScrolling,
							today: today,
							minDate: parseDate(minDate),
							maxDate: parseDate(maxDate),
							theme: theme,
							locale: locale,
							overscanMonthCount: overscanMonthCount
						}))
					)
				)
			);
		}
	}]);
	return InfiniteCalendar;
}(Component);

InfiniteCalendar.defaultProps = {
	width: 400,
	height: 500,
	rowHeight: 56,
	overscanMonthCount: 4,
	layout: 'portrait',
	selectedDate: new Date(),
	min: { year: 1980, month: 0, day: 0 },
	minDate: { year: 1980, month: 0, day: 0 },
	max: { year: 2050, month: 11, day: 31 },
	maxDate: { year: 2050, month: 11, day: 31 },
	keyboardSupport: true,
	autoFocus: true,
	shouldHeaderAnimate: true,
	showOverlay: true,
	showTodayHelper: true,
	showHeader: true,
	tabIndex: 1,
	locale: {},
	theme: {}
};
InfiniteCalendar.propTypes = {
	selectedDate: validDate,
	min: validDate,
	max: validDate,
	minDate: validDate,
	maxDate: validDate,
	locale: PropTypes.object,
	theme: PropTypes.object,
	width: PropTypes.oneOfType([PropTypes.number, PropTypes.string]),
	height: PropTypes.number,
	rowHeight: PropTypes.number,
	className: PropTypes.string,
	overscanMonthCount: PropTypes.number,
	disabledDays: PropTypes.arrayOf(PropTypes.number),
	disabledDates: PropTypes.arrayOf(validDate),
	beforeSelect: PropTypes.func,
	onSelect: PropTypes.func,
	afterSelect: PropTypes.func,
	onScroll: PropTypes.func,
	onScrollEnd: PropTypes.func,
	keyboardSupport: PropTypes.bool,
	autoFocus: PropTypes.bool,
	onKeyDown: PropTypes.func,
	tabIndex: PropTypes.number,
	layout: function layout(props, propName, componentName) {
		if (['portrait', 'landscape'].indexOf(props[propName]) == -1) {
			return new Error('Invalid prop `' + propName + '` supplied to ' + componentName + '. Should be one of `landscape` or `portrait`.');
		}
	},
	shouldHeaderAnimate: PropTypes.bool,
	showOverlay: PropTypes.bool,
	showTodayHelper: PropTypes.bool,
	showHeader: PropTypes.bool
};

var _initialiseProps = function _initialiseProps() {
	var _this2 = this;

	this.onDaySelect = function (selectedDate, e) {
		var _props4 = _this2.props;
		var afterSelect = _props4.afterSelect;
		var beforeSelect = _props4.beforeSelect;
		var onSelect = _props4.onSelect;


		if (!beforeSelect || typeof beforeSelect == 'function' && beforeSelect(selectedDate)) {
			if (typeof onSelect == 'function') {
				onSelect(selectedDate, e);
			}

			_this2.setState({
				selectedDate: selectedDate,
				highlightedDate: selectedDate.clone()
			}, function () {
				_this2.clearHighlight();
				if (typeof afterSelect == 'function') {
					afterSelect(selectedDate);
				}
			});
		}
	};

	this.getCurrentOffset = function () {
		return _this2.scrollTop;
	};

	this.getDateOffset = function (date) {
		return _this2.list && _this2.list.getDateOffset(date);
	};

	this.scrollTo = function (offset) {
		return _this2.list && _this2.list.scrollTo(offset);
	};

	this.scrollToDate = function () {
		var date = arguments.length <= 0 || arguments[0] === undefined ? moment() : arguments[0];
		var offset = arguments[1];

		return _this2.list && _this2.list.scrollToDate(date, offset);
	};

	this.getScrollSpeed = getScrollSpeed();

	this.onScroll = function (_ref) {
		var scrollTop = _ref.scrollTop;
		var _props5 = _this2.props;
		var onScroll = _props5.onScroll;
		var showOverlay = _props5.showOverlay;
		var showTodayHelper = _props5.showTodayHelper;
		var isScrolling = _this2.state.isScrolling;

		var scrollSpeed = _this2.scrollSpeed = Math.abs(_this2.getScrollSpeed(scrollTop));
		_this2.scrollTop = scrollTop;

		// We only want to display the months overlay if the user is rapidly scrolling
		if (showOverlay && scrollSpeed >= 50 && !isScrolling) {
			_this2.setState({
				isScrolling: true
			});
		}

		if (showTodayHelper) {
			_this2.updateTodayHelperPosition(scrollSpeed);
		}
		if (typeof onScroll == 'function') {
			onScroll(scrollTop);
		}
		_this2.onScrollEnd();
	};

	this.onScrollEnd = debounce(function () {
		var _props6 = _this2.props;
		var onScrollEnd = _props6.onScrollEnd;
		var showTodayHelper = _props6.showTodayHelper;
		var isScrolling = _this2.state.isScrolling;


		if (isScrolling) _this2.setState({ isScrolling: false });
		if (showTodayHelper) _this2.updateTodayHelperPosition(0);
		if (typeof onScrollEnd == 'function') onScrollEnd(_this2.scrollTop);
	}, 150);

	this.updateTodayHelperPosition = function (scrollSpeed) {
		if (!_this2.todayOffset) _this2.todayOffset = _this2.getDateOffset(_this2.today.date);
		var scrollTop = _this2.scrollTop;
		var showToday = _this2.state.showToday;

		var newState = void 0;

		if (scrollTop >= _this2.todayOffset + 480) {
			if (showToday !== 1) newState = 1;
		} else if (scrollTop <= _this2.todayOffset - 500) {
			if (showToday !== -1) newState = -1;
		} else if (showToday && scrollSpeed <= 1) {
			newState = false;
		}

		if (scrollTop == 0) {
			newState = false;
		}

		if (newState != null) {
			_this2.setState({ showToday: newState });
		}
	};

	this.handleKeyDown = function (e) {
		var onKeyDown = _this2.props.onKeyDown;
		var _state2 = _this2.state;
		var highlightedDate = _state2.highlightedDate;
		var showToday = _state2.showToday;

		var delta = 0;

		if (typeof onKeyDown == 'function') {
			onKeyDown(e);
		}
		if (!highlightedDate) {
			highlightedDate = _this2.state.selectedDate.clone();
			_this2.setState({ highlightedDate: highlightedDate });
		}

		if ([keyCodes.left, keyCodes.up, keyCodes.right, keyCodes.down].indexOf(e.keyCode) > -1 && typeof e.preventDefault == 'function') {
			e.preventDefault();
		}
		switch (e.keyCode) {
			case keyCodes.enter:
				_this2.setState({
					selectedDate: moment(highlightedDate)
				});
				return;
			case keyCodes.left:
				delta = -1;
				break;
			case keyCodes.right:
				delta = +1;
				break;
			case keyCodes.down:
				delta = +7;
				break;
			case keyCodes.up:
				delta = -7;
				break;
		}

		if (delta) {
			var _props7 = _this2.props;
			var maxDate = _props7.maxDate;
			var minDate = _props7.minDate;
			var rowHeight = _props7.rowHeight;

			var newHighlightedDate = moment(highlightedDate).add(delta, 'days');

			// Make sure the new highlighted date isn't before min / max
			if (newHighlightedDate.isBefore(minDate)) {
				newHighlightedDate = moment(minDate);
			} else if (newHighlightedDate.isAfter(maxDate)) {
				newHighlightedDate = moment(maxDate);
			}

			// Update the highlight indicator
			_this2.clearHighlight();
			var highlightedEl = _this2.highlightedEl = _this2.node.querySelector('[data-date=\'' + newHighlightedDate.format('YYYYMMDD') + '\']');
			if (highlightedEl) {
				highlightedEl.classList.add(style.day.highlighted);
			}

			// Scroll the view
			if (!_this2.currentOffset) _this2.currentOffset = _this2.getCurrentOffset();
			var currentOffset = _this2.currentOffset;
			var monthOffset = _this2.getDateOffset(newHighlightedDate);
			var dateOffset = highlightedEl.offsetTop - rowHeight;
			var newOffset = monthOffset + dateOffset;
			var navOffset = showToday ? 36 : 0;

			if (currentOffset !== newOffset) {
				_this2.currentOffset = newOffset;
				_this2.scrollTo(newOffset - navOffset);
			}

			// Update the reference to the currently highlighted date
			_this2.setState({
				highlightedDate: newHighlightedDate
			});
		}
	};
};

export default InfiniteCalendar;